@@ -470,18 +470,20 @@
 (define (set-cards! slot-id new_cards)
   (set-cards-c! slot-id new_cards))
 
-; Since the cards get thrown around between C and scheme a lot we
-; protect them all from the garbage collector using this guardian.
-; The guardian is defined once. If this file is evaluated again
-; (i.e. at the beginning of a new game) then we clean out the old 
-; cards from the guardian.
-(if (defined? 'card-guardian)
-    (while (card-guardian) #t)
-    (define card-guardian (make-guardian)))
+; Because of the way the cards are passed back and forth to
+; the C code it is possible for references to the cards to be
+; lost. (e.g. when the cards are being moved the list is kept
+; entirely on the C side.) To avoid premature garbage collection 
+; we keep a list of references to the cards.
+;
+; Note that this approach leaks memory since sol.scm is only 
+; re-evaluated and the list emptied when the game has changed.
+; The loss is probably about 1600 bytes per game.
+(define card-ref-list '())
 
 (define (make-card value suit)
   (let ((new-card (list value suit #f)))
-    (card-guardian new-card)
+    (set! card-ref-list (cons new-card card-ref-list))
     new-card))
 
 (define (make-standard-deck-list-ace-high value suit)
